@@ -2,7 +2,7 @@
 #include <ctype.h>
 #include <errno.h>
 #include <glob.h>
-#include <printf.h>
+/*#include <printf.h>*/
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
@@ -113,8 +113,8 @@ const char help[] = "%s"
 " -o --forward      (default for 2UTF) converts to Unicode \n"
 " -H --html                       &<>\" appeared after approximations are escaped\n"
 " -h -? --? -help --help          this help \n"
-" -hp -?p --help=pathnames        outputs various pathnames and directories \n"
 " -l --list-charmaps              lists charmaps & aliases and exits \n"
+" -p --pathnames                  outputs various pathnames and directories \n"
 " -r --reverse      (default for fromUTF) tries convert back to the single bytes \n"
 " -W --show-charmap               shows glyphs in charmap order \n"
 " -S --spit-glyphs                shows glyphs in console font (F000-F1FF) \n"
@@ -157,11 +157,11 @@ const char unknown_opt[] = "unknown option.";
 const char using[] = "using";
 const char will_use[] = "will use";
 int Debug = FALSE, Mail = FALSE, reverse = FALSE, verbose = FALSE;
+ /* stdout may be variable */
 struct charset_type
-  unknown_charset = {stdout, "", NO, UNKNOWN, NULL},
-  USASCII_charset = {stdout, "us-ascii", IS, USASCII, NULL},
-  UTF8_charset = {stdout, "UTF-8", IS, UTF8, NULL}
- ;
+  unknown_charset = {NULL, "", NO, UNKNOWN, NULL},
+  USASCII_charset = {NULL, "us-ascii", IS, USASCII, NULL},
+  UTF8_charset = {NULL, "UTF-8", IS, UTF8, NULL};
 struct charset_type *charset_p = &unknown_charset;
 struct line_buf_type line =
 {NULL, 0};
@@ -174,7 +174,7 @@ ext_charsets[MAX_EXT_CHARSETS + 1];
 wchar_t unknown_wchar = 0xFFFD;
 
 inline int our_wctomb (char *s, unsigned long wc);
-inline int our_mbtowc (unsigned long *p, char *s, unsigned n);
+inline int our_mbtowc (wchar_t *p, char *s, unsigned n);
 
 inline enum charmap_file_format_type
 Charmap_file_format_type (char *pathname)
@@ -200,7 +200,7 @@ Close_pipe (void)
 char *
 Convert (char *to, char *from)
 {
-  register length;
+  register int length;
 
   if (charset_p->type == KNOWN)
     {
@@ -229,7 +229,7 @@ Create_aliases (void)
   glob_t glob_buffer =
   {0, NULL, 0, 0};
   int glob_flags = 0, length;
-  register c, index;
+  register int c, index;
   wchar_t charmap[256], unknown_wchar = 0;
 
   if ((aliases_file = fopen (aliases_pathname, "wt")) == NULL)
@@ -303,7 +303,7 @@ Create_aliases (void)
 	      rewind (charmap_file.stream);
 	      while (fgets (s, max_string_length, charmap_file.stream) != NULL)
 		if (s[0] == '%' && sscanf ((s + 1), " alias %s ", alias) >= 1)
-		  fprintf (aliases_file, "%s ", alias);
+		  fprintf (aliases_file, "%s ", Strtoupper (alias));
 	      fputs (" \n", aliases_file);
 	      if (!feof (charmap_file.stream))
 		fprintf (stderr, "\a2UTF: %s ``%s'' \n", IO_err, \
@@ -336,7 +336,7 @@ Create_aliases (void)
 void
 Error (const char *message)
 {
-  register c;
+  register int c;
 
   fprintf (stderr, "2UTF: %s \n", message);
   if (Mail)
@@ -354,87 +354,91 @@ Fopen_charmap (char *charmap_filename, \
   glob_t glob_buffer =
   {0, NULL, 0, 0};
   int found_alias = FALSE, can_not_create_aliases_file = FALSE;
-  register c, index, length;
+  register int c, index, length;
   size_t s_length = 0;
 
   charmap_file_p->stream = NULL;
   charmap_file_p->format = DEFAULT;
   if (strchr (charmap_filename, '/') != NULL)
-    return (fopen (charmap_filename, "rt"));
-  if ((aliases_file = fopen (aliases_pathname, "rt")) == NULL)
+  {
+    charmap_file_p->stream = fopen (charmap_filename, "rt");
+    charmap_file_p->format = Charmap_file_format_type (charmap_filename);
+  }
+  else {
+    if ((aliases_file = fopen (aliases_pathname, "rt")) == NULL)
     {
       fprintf (stderr, "2UTF: %s %s \n", can_not_open, aliases_pathname);
       if (!Create_aliases ())
-	{
-	  can_not_create_aliases_file = 1;
-	  fprintf (stderr, "2UTF: %s ``%s'' \n", can_not_create, \
-		   aliases_pathname);
-	}
+      {
+	can_not_create_aliases_file = 1;
+	fprintf (stderr, "2UTF: %s ``%s'' \n", can_not_create, \
+	    aliases_pathname);
+      }
       else if ((aliases_file = fopen (aliases_pathname, "rt")) == NULL)
 	Error (internal_err);
     }
-  charmap_name = xmalloc (strlen (charmap_filename) + 5);
-  if (Debug >= 9)
-    fprintf (stderr, "2UTF: malloc (%i) \n", (strlen (charmap_filename) + 5));
-  Strtoupper (\
-       strcat (strcat (strcpy (charmap_name, " "), charmap_filename), " "));
-  if (aliases_file != NULL)
+    charmap_name = xmalloc (strlen (charmap_filename) + 5);
+    if (Debug >= 9)
+      fprintf (stderr, "2UTF: malloc (%i) \n", (int)(strlen (charmap_filename) + 5));
+    Strtoupper (\
+	strcat (strcat (strcpy (charmap_name, " "), charmap_filename), " "));
+    if (aliases_file != NULL)
     {
       for (index = 0; index < 2; index++)
-	{
-	  if (Debug >= 3)
-	    fprintf (stderr, "2UTF: Looking for alias ``%s'' \n", charmap_name);
-	  while (Getline ((unsigned char **) &s, &s_length, aliases_file) \
-	      != (size_t) -1)
-	    if ((strchr (s, ' ') != NULL) && \
-		(Str_str (strchr (s, ' '), charmap_name) != NULL))
-	      {
-		strtok (s, "\n");
-		if (!found_alias)
-		  {
-		    if (verbose)
-		      fprintf (stderr, "2UTF: %s - %s: \n   ``%s'' \n",
-			       charmap_name, will_use, s);
-		    charmap_file_p->stream = fopen (ptr = strtok (s, " "), "rt");
-		    charmap_file_p->format = Charmap_file_format_type (ptr);
-		  }
-		else if (verbose)
-		  fprintf (stderr, "2UTF: %s - %s: \n   ``%s'' \n",
-			   charmap_name, another_match, s);
-		else
-		  fprintf (stderr, "2UTF: %s. \n", another_match);
-		found_alias = 1;
-	      }
-	  if (!feof (aliases_file))
-	    if (ferror (aliases_file))
-	      Error (IO_err);
-	    else
-	      Error (out_of_mem);
-	  if (!found_alias)
+      {
+	if (Debug >= 3)
+	  fprintf (stderr, "2UTF: Looking for alias ``%s'' \n", charmap_name);
+	while (Getline ((unsigned char **) &s, &s_length, aliases_file) \
+	    != (size_t) -1)
+	  if ((strchr (s, ' ') != NULL) && \
+	      (Str_str (strchr (s, ' '), charmap_name) != NULL))
+	  {
+	    strtok (s, "\n");
+	    if (!found_alias)
 	    {
-	      if (index == 0)
-		{
-		  clearerr (aliases_file);
-		  rewind (aliases_file);
-		  memmove (charmap_name, &charmap_name[1], \
-			   strlen (charmap_name) - 1);
-		  strtok (charmap_name, " ");
-		}
+	      if (verbose)
+		fprintf (stderr, "2UTF: %s - %s: \n   ``%s'' \n",
+		    charmap_name, will_use, s);
+	      charmap_file_p->stream = fopen (ptr = strtok (s, " "), "rt");
+	      charmap_file_p->format = Charmap_file_format_type (ptr);
 	    }
+	    else if (verbose)
+	      fprintf (stderr, "2UTF: %s - %s: \n   ``%s'' \n",
+		  charmap_name, another_match, s);
+	    else
+	      fprintf (stderr, "2UTF: %s. \n", another_match);
+	    found_alias = 1;
+	  }
+	if (!feof (aliases_file))
+	  if (ferror (aliases_file))
+	    Error (IO_err);
 	  else
-	    break;
+	    Error (out_of_mem);
+	if (!found_alias)
+	{
+	  if (index == 0)
+	  {
+	    clearerr (aliases_file);
+	    rewind (aliases_file);
+	    memmove (charmap_name, &charmap_name[1], \
+		strlen (charmap_name) - 1);
+	    strtok (charmap_name, " ");
+	  }
 	}
+	else
+	  break;
+      }
       free (s);
       if (EOF == fclose (aliases_file))
 	Error (IO_err);
     }
-  if (!found_alias || charmap_file_p->stream == NULL)
+    if (!found_alias || charmap_file_p->stream == NULL)
     {
       if (charmap_name[0] == ' ')
-	{
-	  memmove (charmap_name, &charmap_name[1], strlen (charmap_name) - 1);
-	  strtok (charmap_name, " ");
-	}
+      {
+	memmove (charmap_name, &charmap_name[1], strlen (charmap_name) - 1);
+	strtok (charmap_name, " ");
+      }
       if (Debug >= 3)
 	fprintf (stderr, "2UTF: Looking for file ``%s'' \n", charmap_name);
       for (length = strlen (paths[0]), index = 1; index < paths_number; index++)
@@ -443,84 +447,85 @@ Fopen_charmap (char *charmap_filename, \
       pathname = xmalloc (length + strlen (charmap_name) + 5);
       if (Debug >= 9)
 	fprintf (stderr, "2UTF: malloc (%i) \n", \
-		 (length + strlen (charmap_name) + 5));
+	    (int)(length + strlen (charmap_name) + 5));
       for (index = 0; index < paths_number; index++)
+      {
+	strcat (strcpy (pathname, paths[index]), charmap_name);
+	if (Debug >= 3)
+	  fprintf (stderr, "2UTF: Looking for ``%s'' \n", pathname);
+	if ((charmap_file_p->stream = fopen (pathname, "rt")) != NULL)
 	{
-	  strcat (strcpy (pathname, paths[index]), charmap_name);
+	  charmap_file_p->format = Charmap_file_format_type (pathname);
+	  if (verbose)
+	    fprintf (stderr, "2UTF: %s ``%s'' \n", using, pathname);
+	  break;
+	}
+      }
+      if (charmap_file_p->stream == NULL)
+      {
+	for (index = 0; index < paths_number; index++)
+	{
+	  strcat (strcpy (pathname, paths[index]), "*");
+	  strcat (strcat (pathname, charmap_name), "*");
 	  if (Debug >= 3)
 	    fprintf (stderr, "2UTF: Looking for ``%s'' \n", pathname);
-	  if ((charmap_file_p->stream = fopen (pathname, "rt")) != NULL)
-	    {
-	      charmap_file_p->format = Charmap_file_format_type (pathname);
-	      if (verbose)
-		fprintf (stderr, "2UTF: %s ``%s'' \n", using, pathname);
-	      break;
-	    }
-	}
-      if (charmap_file_p->stream == NULL)
-	{
-	  for (index = 0; index < paths_number; index++)
-	    {
-	      strcat (strcpy (pathname, paths[index]), "*");
-	      strcat (strcat (pathname, charmap_name), "*");
-	      if (Debug >= 3)
-		fprintf (stderr, "2UTF: Looking for ``%s'' \n", pathname);
-	      /*c = glob (pathname, GLOB_NOSORT, NULL, &glob_buffer); */
-	      c = glob (pathname, 0, NULL, &glob_buffer);
-	      switch (c)
+	  /*c = glob (pathname, GLOB_NOSORT, NULL, &glob_buffer); */
+	  c = glob (pathname, 0, NULL, &glob_buffer);
+	  switch (c)
+	  {
+	    case 0:
+	      if (glob_buffer.gl_pathc >= 1)
+	      {
+		if (glob_buffer.gl_pathc >= 2)
+		  fprintf (stderr, "2UTF: %s ``%s'' (%i). \n", \
+		      multiply_matches, pathname, glob_buffer.gl_pathc);
+		if ((charmap_file_p->stream = \
+		    fopen (glob_buffer.gl_pathv[0], "rt")) != NULL)
 		{
-		case 0:
-		  if (glob_buffer.gl_pathc >= 1)
-		    {
-		      if (glob_buffer.gl_pathc >= 2)
-			fprintf (stderr, "2UTF: %s ``%s'' (%i). \n", \
-			  multiply_matches, pathname, glob_buffer.gl_pathc);
-		      if ((charmap_file_p->stream = \
-			   fopen (glob_buffer.gl_pathv[0], "rt")) != NULL)
-			{
-			  charmap_file_p->format = \
-			    Charmap_file_format_type (glob_buffer.gl_pathv[0]);
-			  if (glob_buffer.gl_pathc != 1)
-			    fprintf (stderr, "2UTF: %s ``%s'' \n", using, \
-				     glob_buffer.gl_pathv[0]);
-			  goto break_for;
-			}
-		      else if (verbose)
-			fprintf (stderr, "2UTF: %s ``%s'' \n", can_not_open, \
-				 glob_buffer.gl_pathv[0]);
-		    }
-		  break;
-		case GLOB_NOSPACE:
-		  Error (out_of_mem);
-		default:
-		  if (Debug >= 5)
-		    fprintf (stderr, "2UTF: glob() returned %i \n", c);
+		  charmap_file_p->format = \
+		    Charmap_file_format_type (glob_buffer.gl_pathv[0]);
+		  if (glob_buffer.gl_pathc != 1)
+		    fprintf (stderr, "2UTF: %s ``%s'' \n", using, \
+			glob_buffer.gl_pathv[0]);
+		  goto break_for;
 		}
-	    }
-	break_for:
-	  globfree (&glob_buffer);
+		else if (verbose)
+		  fprintf (stderr, "2UTF: %s ``%s'' \n", can_not_open, \
+		      glob_buffer.gl_pathv[0]);
+	      }
+	      break;
+	    case GLOB_NOSPACE:
+	      Error (out_of_mem);
+	    default:
+	      if (Debug >= 5)
+		fprintf (stderr, "2UTF: glob() returned %i \n", c);
+	  }
 	}
+break_for:
+	globfree (&glob_buffer);
+      }
       free (pathname);
       free (charmap_name);
       if (!found_alias && verbose && charmap_file_p->stream != NULL)
 	fprintf (stderr, "2UTF: %s \n", needs_update);
     }
-  if (found_alias && charmap_file_p->stream == NULL)
-    fprintf (stderr, "2UTF: %s \n", needs_update);
-  if (((found_alias && charmap_file_p->stream == NULL) || \
-       (!found_alias && charmap_file_p->stream != NULL)) \
-      &&strcmp (charmap_format[0], default_charmap_format) == 0)
-    if (!can_not_create_aliases_file)
-      if (!Create_aliases ())
+    if (found_alias && charmap_file_p->stream == NULL)
+      fprintf (stderr, "2UTF: %s \n", needs_update);
+    if (((found_alias && charmap_file_p->stream == NULL) || \
+	(!found_alias && charmap_file_p->stream != NULL)) \
+	&&strcmp (charmap_format[0], default_charmap_format) == 0)
+      if (!can_not_create_aliases_file)
+	if (!Create_aliases ())
 	{
 	  if (verbose)
 	    fprintf (stderr, "2UTF: %s ``%s'' \n", can_not_create, \
-		     aliases_pathname);
+		aliases_pathname);
 	}
-      else if ((aliases_file = fopen (aliases_pathname, "rt")) == NULL)
-	Error (internal_err);
-      else if (EOF == fclose (aliases_file))
-	Error (IO_err);
+	else if ((aliases_file = fopen (aliases_pathname, "rt")) == NULL)
+	  Error (internal_err);
+	else if (EOF == fclose (aliases_file))
+	  Error (IO_err);
+  }
   return (charmap_file_p->stream);
 }
 
@@ -536,7 +541,7 @@ inline int
 Freadmb (wchar_t * wchar_p, FILE * stream)
 {
   char mb[6];
-  register length, i, c;
+  register int length, i, c;
 
   if ((c = getc (stream)) == EOF)
     return (EOF);
@@ -924,7 +929,7 @@ Parse_string (char *charmap_format_string, wchar_t * charmap, char *s)
       charmap[index] = (wchar_t) unichar;
       if (Debug > 8)
 	fprintf (stderr, "2UTF: ``%s''  -->   0x%.2X is U+%.4lX.\n" \
-		 ,s, index, charmap[index]);
+		 ,s, index, (long unsigned int) charmap[index]);
       return (1);
     }
   else
@@ -934,7 +939,7 @@ Parse_string (char *charmap_format_string, wchar_t * charmap, char *s)
 inline void
 Pipe_to_UTF8 (FILE * in_stream)
 {
-  register c;
+  register int c;
 
   if (charset_p->type == KNOWN)
     while ((c = getc (in_stream)) != EOF)
@@ -951,7 +956,7 @@ inline int
 Putc_in_UTF8 (int c, FILE * out_stream)
 {
   char s[12];
-  register length;
+  register int length;
 
   length = wctomb (s, charset_p->charmap[c]);
   if (length != -1)
@@ -1228,7 +1233,7 @@ Str_has_prefix (const char *string, const char *prefix)
 int
 Str_is_UTF8 (char *charmap_filename)
 {
-  register i, iplus;
+  register int i, iplus;
 
 /* Unicode-1-1-UTF-8 */
   if (Strcasecmp (charmap_filename, "UTF-8") == 0 || \
@@ -1259,7 +1264,7 @@ Str_is_boundary (char *linebuf, char *boundary, int push_boundary)
     }
   boundaries =
   {
-    NULL, -1, -1
+    NULL, -1, -1, 0
   };
   unsigned char *lbp;
 
@@ -1329,7 +1334,7 @@ Str_str (char *haystack_, char *needle_)
 {
   char *haystack, *needle;
   register char *n_, *n;
-  register index;
+  register int index;
 
   haystack = xstrdup (haystack_);
   needle = xstrdup (needle_);
@@ -1355,7 +1360,7 @@ Str_str (char *haystack_, char *needle_)
 inline char *
 Strtoupper (char *string)
 {
-  register index;
+  register int index;
 
   for (index = 0; string[index] != 0; index++)
     string[index] = toupper (string[index]);
@@ -1367,10 +1372,10 @@ Spit (wchar_t min, wchar_t max, wchar_t charmap[], char unknown_char)
 {
   char s[22], header[] = "  0 1 2 3 4 5 6 7 8 9 A B C D E F  0 1 2 3 4 5 6 7 8 9 A B C D E F \n";
   register wchar_t index;
-  register length;
+  register int length;
   int prefix_length;
 
-  prefix_length = sprintf (s, "%.2lx", max);
+  prefix_length = sprintf (s, "%.2lx", (long unsigned int) max);
   fprintf (charset_p->pipe, "%*c%s", prefix_length, ' ', header);
   for (index = min / 32 * 32; index <= max / 32 * 32 + 31; index++)
     {
@@ -1612,7 +1617,7 @@ main (int argc, char *argv[])	/* char *env[]) */
   char unknown_char = default_unknown_char;
   char *charmap_filename = NULL, *ptr;
   unsigned char *next_token, /* s[max_string_length], */ *stop_char_ptr;
-  const char *optstring = "248Cc:d::f::Hh::lorS::su::vVWw";
+  const char *optstring = "248Cc:d::f::HhloprS::su::vVWw";
   const struct long2short_options_type long2short_options[] =
   {
     {
@@ -1771,7 +1776,7 @@ main (int argc, char *argv[])	/* char *env[]) */
     swap, switch_to_unicode = FALSE;
 
   /*register length, c; */
-  register c;
+  register int c;
   long int l;
   union
     {
@@ -1782,6 +1787,8 @@ main (int argc, char *argv[])	/* char *env[]) */
   u_int;
   wchar_t range, spit_min = 0xF000, spit_max = 0xF1FF, wchar;
 
+  UTF8_charset.pipe = USASCII_charset.pipe = unknown_charset.pipe = stdout;
+
 /* standard UCS byte order is big endian */
 #if (__BYTE_ORDER != __BIG_ENDIAN)
   swap = TRUE;
@@ -1848,13 +1855,7 @@ main (int argc, char *argv[])	/* char *env[]) */
 	      html_doc = TRUE;
 	      break;
 	    case 'h':
-	      if (optarg && optarg[0] == 'p')
-		{
-		  Read_config_file ();
-		  Print_paths ();
-		}
-	      else
-		Help (default_unknown_char);
+	      Help (default_unknown_char);
 	      exit (0);
 	    case 'l':
 	      list_charmaps = TRUE;
@@ -1862,6 +1863,10 @@ main (int argc, char *argv[])	/* char *env[]) */
 	    case 'o':
 	      reverse = FALSE;
 	      break;
+	    case 'p':
+	      Read_config_file ();
+	      Print_paths ();
+	      exit (0);
 	    case 'r':
 	      reverse = TRUE;
 	      break;
@@ -1916,7 +1921,8 @@ main (int argc, char *argv[])	/* char *env[]) */
 		}
 /*fprintf(stderr,"min = %lx\n",spit_min); */
 	      if (verbose)
-		fprintf (stderr, "0x%.4lX-0x%.4lX\n", spit_min, spit_max);
+		fprintf (stderr, "0x%.4lX-0x%.4lX\n", \
+		    (long unsigned int) spit_min, (long unsigned int) spit_max);
 	      break;
 	    case 's':
 	      switch_to_unicode = TRUE;
@@ -2039,6 +2045,19 @@ main (int argc, char *argv[])	/* char *env[]) */
       if (verbose)
 	fprintf (stderr, "2UTF: %s ``%s'' \n", using, "Unicode UTF-8");
     }
+  else if (charset_p->type == USASCII)
+    {
+      if (verbose)
+	fprintf (stderr, "2UTF: %s ``%s'' \n", using, "US-ASCII");
+      charset_p->charmap = xmalloc (256 * sizeof (wchar_t));
+      charset_p->type = KNOWN;
+      charset_p->pipe = stdout;
+      charset_p->USASCII_is_subset = IS;
+      for (wchar = 0; wchar <= 0x7F; wchar++)
+	charset_p->charmap[wchar] = wchar;
+      for (wchar = 0x7F; wchar <= 0xFF; wchar++)
+	charset_p->charmap[wchar] = unknown_wchar;
+    }
 
   if (charmap_filename != argv[file_arg])
     free (charmap_filename);
