@@ -137,7 +137,7 @@ parse_message (boundary)
 	  /* Check if part's own headers are missing */
 	  if ((c = getc (stdin)) == '\n')
 	    {
-	      putchar (c);
+	      putc (c, charset_p->pipe);
 	      if (mhead.content_type == MULT_DIGEST)
 		/* default is message/rfc */
 		ret = parse_message (mhead.boundary);
@@ -280,8 +280,8 @@ Add_to_buffer (struct buffer_type *buffer_p, char *string)
   length = strlen (string);
   if (buffer_p->next + length - buffer_p->end > reserve)
     {
-      fprintf (stderr, "2UTF: %s %i\n", buffer_overflow, \
-	       buffer_p->next + length - buffer_p->end - reserve);
+      fprintf (stderr, "2UTF: %s %li\n", buffer_overflow, \
+	       (long)(buffer_p->next + length - buffer_p->end - reserve));
       length = buffer_p->end - buffer_p->next - reserve;
     }
   memcpy (buffer_p->next, string, length);
@@ -345,8 +345,8 @@ parse_header (mhp)
 	  headers_buffer.header_start - headers_buffer.start;
 
 	  if (Debug >= 9)
-	    fprintf (stderr, "2UTF: realloc %i += %i \n", \
-		     old_size + reserve, REALLOC_CHUNK);
+	    fprintf (stderr, "2UTF: realloc %li += %i \n", \
+		     (long)old_size + reserve, REALLOC_CHUNK);
 	  line.buf = headers_buffer.start = xrealloc (headers_buffer.start, \
 					old_size + reserve + REALLOC_CHUNK);
 	  line.length += REALLOC_CHUNK;
@@ -355,8 +355,8 @@ parse_header (mhp)
 	  headers_buffer.header_start = \
 	    headers_buffer.start + header_start_offset;
 	  if (line.length != old_size + reserve + REALLOC_CHUNK && Debug)
-	    fprintf (stderr, "2UTF: memory allocation error: \n  %i != %i \n", \
-		     line.length, old_size + reserve + REALLOC_CHUNK);
+	    fprintf (stderr, "2UTF: memory allocation error: \n  %li != %li \n", \
+		     (long)line.length, (long)old_size + reserve + REALLOC_CHUNK);
 	}
       if ((c > 0x7F || (c != '\n' && c != '\t' && iscntrl (c))) \
 	  &&headers_8bit == FALSE)
@@ -1428,6 +1428,7 @@ decode_base64 (infp, outfp, boundary)
 	{
 	  if (Debug >= 2)
 	    fprintf (stderr, " --- boundary line prefix found: %d\n", ret);
+	  putc ('\n', outfp);/*7*/
 	  putc ('-', outfp);
 	  putc ('-', outfp);
 	  if (!boundary || -1 == \
@@ -1464,6 +1465,7 @@ decode_base64 (infp, outfp, boundary)
     }
   if (Debug >= 2)
     fprintf (stderr, " --- end of base64 \n");
+  putc ('\n', outfp);/*7*/
   if (boundary)
     return (Seek_boundary (infp, boundary, 0));
   else
